parse_unchecked

Function parse_unchecked 

pub fn parse_unchecked(source: &str, options: ParseOptions) -> Parsed<Mod>
Parse the given Python source code using the specified ParseOptions.

This is same as the parse function except that it doesn’t check for any ParseError and returns the Parsed as is.