Module semantic_errors

Module semantic_errors 

SemanticSyntaxChecker for AST-based syntax errors.

This checker is not responsible for traversing the AST itself. Instead, its SemanticSyntaxChecker::visit_stmt and SemanticSyntaxChecker::visit_expr methods should be called in a parent Visitor’s visit_stmt and visit_expr methods, respectively.

Structs

SemanticSyntaxChecker
SemanticSyntaxError

Enums

AwaitOutsideAsyncFunctionKind
InvalidExpressionKind
InvalidExpressionPosition
SemanticSyntaxErrorKind
WriteToDebugKind
YieldOutsideFunctionKind

Traits

SemanticSyntaxContext
Information needed from a parent visitor to emit semantic syntax errors.