pub enum Mode {
Module,
Expression,
ParenthesizedExpression,
Ipython,
}Expand description
Control in the different modes by which a source file can be parsed.
The mode argument specifies in what way code must be parsed.
Variants
Module
The code consists of a sequence of statements.
Expression
The code consists of a single expression.
ParenthesizedExpression
The code consists of a single expression and is parsed as if it is parenthesized. The parentheses themselves aren’t required. This allows for having valid multiline expression without the need of parentheses and is specifically useful for parsing string annotations.
Ipython
The code consists of a sequence of statements which can include the escape commands that are part of IPython syntax.
Supported escape commands:
- Magic command system which is limited to line magics and can start
with
?or??. - Dynamic object information which can start with
?or??. - System shell access which can start with
!or!!. - Automatic parentheses and quotes which can start with
/,;, or,.